@@ -18,22 +18,9 @@
 """Previewing subtitles with a video player."""
 
 import aeidon
-import mimetypes
 import os
 import string
 
-VIDEOFILE_EXTENSIONS = [
-    ".avi",
-    ".flv",
-    ".mkv",
-    ".mov",
-    ".mp4",
-    ".ogg",
-    ".ogv",
-    ".vob",
-    ".webm",
-]
-
 
 class PreviewAgent(aeidon.Delegate):
 
@@ -61,11 +48,7 @@ class PreviewAgent(aeidon.Delegate):
             path = os.path.join(dirname, name)
             root = os.path.splitext(name)[0]
             if not subname.startswith(root): continue
-            # The mimetypes module doesn't work well on Windows,
-            # fall back on a custom list of videfile extensions.
-            type, encoding = mimetypes.guess_type(path)
-            if ((type and type.startswith("video/")) or
-                path.lower().endswith(tuple(VIDEOFILE_EXTENSIONS))):
+            if aeidon.util.is_video_file(path):
                 self.video_path = path
                 return self.video_path
         return None
